/* simplestring.cc
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Matthew Stanton <matthews@softhome.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 *
 * A simple implementation of the Karplus-Strong plucked string physical model
 * with damping.
 *
 */

#define OX_CLASS SimpleString
#include "ox_wrappers.h"
#include "simplestring.h"
#include "ox_wrappers.cc"


/********************************
  SimpleString
********************************/

SimpleString::SimpleString(machine* m) : OX_Machine(m) {
	vol = 0x80;
	pitch = 0.0;
	buflen = 0;
	bufpos = 0;
	buf = new float[BUFFER_SIZE];
	lastout = 0.0;
	damper = 0.99506;
	
	for (int i=0; i<BUFFER_SIZE; i++) {
		buf[i] = 0;
	}
}

SimpleString::~SimpleString() {
	delete buf;
}

int SimpleString::initialize(machine_type* t) {

	static param_spec my_params[] = {
		{
			OX_FORMAT_NOTE,
			OX_WIDGET_SLIDER,
			"note",
			"Which note to play.",
			0,
			NOTE_RANGE,
			0
		},
		
		{
			OX_FORMAT_SMALL,
			OX_WIDGET_SLIDER,
			"vol",
			"Volume: 00-FF",
			0x00,
			0xFF,
			0x80
		},
		
		{
			OX_FORMAT_SMALL,
			OX_WIDGET_SLIDER,
			"damper",
			"Damper amount",
			0x00,
			0x80,
			0x6E
		},
		
		{
			OX_FORMAT_TRIGGER,
			OX_WIDGET_BUTTON,
			"pluck",
			"Pluck the string",
			0,
			1,
			0
		}
	};
	
	t->long_name		= "Serf SimpleString";
	t->short_name		= "sstring";
	t->max_tracks		= 1;
	t->input_channels	= 0;
	t->output_channels	= 1;
	t->num_params		= my_num_params;
	t->param_specs		= my_params;
	
	return(1);
}

void SimpleString::describe(char* dest, int which, param value) {
	float x;
	int percent;
	
	switch(which) {
	case ix_note:
		sprintf(dest, "$");
		break;
		
	case ix_vol:
		x = ((float)value) / 255.0;
		percent = (int)(x*100);
		sprintf(dest, "%d%%", percent);
		break;
		
	case ix_damper:
		sprintf(dest, "%5.5f", scaledamper((float)(value / 128.0)));
		break;

	case ix_pluck:
		sprintf(dest, "Gate toggled.");
		break;
	
	default:
		sprintf(dest, "ERROR");
	}
}

void SimpleString::update(int track, int which, param value) {
	switch(which) {
	case ix_note:
		pitch = m->pkg->note2freq(value);
		break;
		
	case ix_vol:
		vol = ((samp)value) / 255.0;
		break;
	
	case ix_damper:
		damper = scaledamper(((float)value) / 128.0);
		break;
	
	case ix_pluck:
		pluck();
		break;
	}
}

int SimpleString::work(int block_size) {
	samp out;
	
	if (buflen == 0) return 0;
	
	for (int i=0; i < block_size; i++) {
		out = buf[bufpos];
		buf[bufpos] = ((out + lastout) * damper) / 2;
		
		lout[i] = out * vol;
		lastout = out;
		
		if (++bufpos >= buflen) bufpos = 0;	// wrap
	}

	return(1);
}

int SimpleString::track(int change) {
	return(0);  // Not polyphonic yet.
}

void SimpleString::pluck() {
	// calculate buffer size for new pitch, then sanity check
	buflen = (int)(SAMPLE_RATE / pitch);
	if (buflen > BUFFER_SIZE) buflen = BUFFER_SIZE;
	if (buflen < 0) buflen = 0;
	
	// fill buffer with white noise and send pointer to start of buffer
	for (int i=0; i<buflen; i++) {
		buf[i] = 2.0 * ((float)rand() / RAND_MAX) - 1.0;
	}
	bufpos = 0;
}

float SimpleString::scaledamper(float value) {
	// same scaling as Geonik's
	return (1.0 - ((1.0-value)*(1.0-value)/4.0));
}
